// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package model

import (
	"adm-ads/common.in/utils"
	"go.uber.org/zap"
	"gorm.io/gorm"
	"time"
)

type Ads14 struct {
	ID        int64  `gorm:"primary_key" json:"-"`
	StyleId string `json:"style_id"`
	C2Id int `json:"c2_id"`
	AttributeRule string `json:"attribute_rule"`
	Useage string `json:"useage"`
	Detail string `json:"detail"`
	FrontWheelSpecification string `json:"front_wheel_specification" gorm:"-"`
	BackWheelSpecification string `json:"back_wheel_specification" gorm:"-"`
	//TyreSize                           string `json:"tyre_size" gorm:"-"`
	CreatedAt int64  `json:"-"`
	UpdatedAt int64  `json:"-"`
}

func (Ads14) TableName() string {
	return "t_adm_ads14"
}

// Insert 插入一条记录
func (p *Ads14) Insert(db *gorm.DB) error {
	if p.CreatedAt == 0 {
		timeNow := time.Now().Unix()
		p.CreatedAt = timeNow
		p.UpdatedAt = timeNow
	}
	err := db.Create(p).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(*p)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Ads14) InsertByMap(db *gorm.DB, data map[string]interface{}) error {
	err := db.Model(p).Create(data).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(data)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Ads14) Delete(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Delete(p).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "delete from "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(filter)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Ads14) Save(db *gorm.DB) error {
	err := db.Save(p).Error
	if err != nil {

		l.Error("mysql",
			zap.String("sql", "save "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(*p)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Ads14) Update(db *gorm.DB) error {
	p.CreatedAt = 0
	if p.UpdatedAt == 0 {
		timeNow := time.Now().Unix()
		p.UpdatedAt = timeNow
	}
	err := db.Model(p).UpdateColumns(p).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(*p)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Ads14) UpdateByMap(db *gorm.DB, data map[string]interface{}) error {
	err := db.Model(p).Updates(data).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(data)),
			zap.String("error", err.Error()))
	}
	return err
}

// 通过结构体变量更新字段值, gorm库会忽略零值字段。就是字段值等于0, nil, "", false这些值会被忽略掉，不会更新。如果想更新零值，可以使用map类型替代结构体。
func (p *Ads14) UpdateSome(db *gorm.DB, filed map[string]interface{}) error {
	err := db.Model(p).Updates(filed).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(filed)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Ads14) Query(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Find(p).Error
	if err != nil {

		l.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(filter)),
			zap.String("error", err.Error()))
	}

	if p.ID == 0 {
		return gorm.ErrRecordNotFound
	}

	return err
}

func (p *Ads14) QueryAll(db *gorm.DB, filter map[string]interface{}, out interface{}) error {
	err := db.Where(filter).Find(out).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(filter)),
			zap.String("error", err.Error()))
	}
	return err
}
