// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package model

import (
	"adm-ads/common.in/utils"
	"go.uber.org/zap"
	"gorm.io/gorm"
	"time"
)

type Ads9 struct {
	ID        int64  `gorm:"primary_key" json:"-"`
	VinRule string `json:"vin_rule"`
	ModelNo string `json:"model_no"`
	ThirdSeriesId string `json:"third_series_id"`
	SeriesId string `json:"series_id"`
	ModelYear string `json:"model_year"`
	DisplacementL  string `json:"displacement_l"`
	Vin10 string `json:"vin10"`
	AirIntakForm string `json:"air_intak_form"`
	Uniq string `json:"uniq"`
	Source string `json:"source"`
	CreatedAt int64  `json:"-"`
	UpdatedAt int64  `json:"-"`
}

func (Ads9) TableName() string {
	return "t_adm_ads9"
}

// Insert 插入一条记录
func (p *Ads9) Insert(db *gorm.DB) error {
	if p.CreatedAt == 0 {
		timeNow := time.Now().Unix()
		p.CreatedAt = timeNow
		p.UpdatedAt = timeNow
	}
	err := db.Create(p).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(*p)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Ads9) InsertByMap(db *gorm.DB, data map[string]interface{}) error {
	err := db.Model(p).Create(data).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(data)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Ads9) Delete(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Delete(p).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "delete from "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(filter)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Ads9) Save(db *gorm.DB) error {
	err := db.Save(p).Error
	if err != nil {

		l.Error("mysql",
			zap.String("sql", "save "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(*p)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Ads9) Update(db *gorm.DB) error {
	p.CreatedAt = 0
	if p.UpdatedAt == 0 {
		timeNow := time.Now().Unix()
		p.UpdatedAt = timeNow
	}
	err := db.Model(p).UpdateColumns(p).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(*p)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Ads9) UpdateByMap(db *gorm.DB, data map[string]interface{}) error {
	err := db.Model(p).Updates(data).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(data)),
			zap.String("error", err.Error()))
	}
	return err
}

// 通过结构体变量更新字段值, gorm库会忽略零值字段。就是字段值等于0, nil, "", false这些值会被忽略掉，不会更新。如果想更新零值，可以使用map类型替代结构体。
func (p *Ads9) UpdateSome(db *gorm.DB, filed map[string]interface{}) error {
	err := db.Model(p).Updates(filed).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(filed)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Ads9) Query(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Find(p).Error
	if err != nil {

		l.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(filter)),
			zap.String("error", err.Error()))
	}

	if p.ID == 0 {
		return gorm.ErrRecordNotFound
	}

	return err
}

func (p *Ads9) QueryAll(db *gorm.DB, filter map[string]interface{}, out interface{}) error {
	err := db.Where(filter).Find(out).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(filter)),
			zap.String("error", err.Error()))
	}
	return err
}
