// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package model

import (
	"adm-ads/common.in/utils"
	"gorm.io/gorm"
	"go.uber.org/zap"
	"time"
)

type TaskRelation struct {
	ID       int64  `gorm:"primary_key"`
	TaskId   int `json:"task_id"`
	MsgType  string `json:"msg_type"`
	CreatedAt int64 `json:"created_at"`
	UpdatedAt int64 `json:"updated_at"`
}

func (TaskRelation) TableName() string {
	return "t_adm_ods_task_relation"
}

// Insert 插入一条记录
func (p *TaskRelation) Insert(db *gorm.DB) error {
	timeNow := time.Now().Unix()
	p.CreatedAt = timeNow
	p.UpdatedAt = timeNow
	err := db.Create(p).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(*p)),
			zap.String("error", err.Error()))
	}
	return err
}


func (p *TaskRelation)InsertByMap(db *gorm.DB,data map[string]interface{}) error {
	data["updated_at"] = time.Now().Unix()
	err := db.Model(p).Create(data).Error
	if err != nil{
		l.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(data)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TaskRelation) Delete(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Delete(p).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "delete from "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(filter)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TaskRelation) Save(db *gorm.DB) error {
	timeNow := time.Now().Unix()
	p.UpdatedAt = timeNow
	//db.Model().Update()
	err := db.Save(p).Error
	if err != nil {

		l.Error("mysql",
			zap.String("sql", "save "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(*p)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TaskRelation) Update(db *gorm.DB) error {
	timeNow := time.Now().Unix()
	p.UpdatedAt = timeNow
	err := db.Model(p).Updates(p).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(*p)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TaskRelation)UpdateByMap(db *gorm.DB,data map[string]interface{}) error {
	data["updated_at"] = time.Now().Unix()
	err := db.Model(p).Updates(data).Error
	if err != nil{
		l.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(data)),
			zap.String("error", err.Error()))
	}
	return err
}

// 通过结构体变量更新字段值, gorm库会忽略零值字段。就是字段值等于0, nil, "", false这些值会被忽略掉，不会更新。如果想更新零值，可以使用map类型替代结构体。
func (p *TaskRelation) UpdateSome(db *gorm.DB, filed map[string]interface{}) error {
	/*if filed == nil {
		return errors.ParamsError
	}*/
	timeNow := time.Now().Unix()
	filed["updated_at"] = timeNow
	err := db.Model(p).Updates(filed).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(filed)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TaskRelation) Query(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Find(p).Error
	if err != nil {

		l.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(filter)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *TaskRelation) QueryAll(db *gorm.DB, filter map[string]interface{}, out interface{}) error {
	err := db.Where(filter).Find(out).Error
	if err != nil {
		l.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", utils.MarshalJsonString(filter)),
			zap.String("error", err.Error()))
	}
	return err
}
